
package grammars;

/**
 * Trida slouzici k reprezentaci derivace. Pamatuje si aktualni vetnou formu
 * a gramatiku. Umoznuje navrat v derivaci zpet. Derivace neni vazana na
 * konkretni typ gramatiky zdedene ze tridy Grammar.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class Derivation {
  /**Gramatika pouzita pro derivaci*/
  protected Grammar grammar;
  /**Aktualni vetna forma*/
  protected SententialForm form;
  /**Kroky derivace*/
  protected Step[] steps;
  
  /**
   * Konstruktor tridy se specifikaci pouzite gramatiky. Vetna forma bude 
   * tvorena pocatecnim neterminalem gramatiky.
   *
   * @param g Gramatika pouzivana k derivaci
   */  
  public Derivation(Grammar g) {
    this.grammar=g;
    this.form=new SententialForm(grammar.getStart());
    this.steps=new Step[0];
  }

  /**
   * Navrat v derivaci o jeden krok zpet. Tento krok je zapomenut.
   */  
  public void backOneStep(){
    if(steps.length==0){
      return;
    }
    Step newSteps[]=new Step[steps.length-1];
    int i;
    for(i=0;i<steps.length-1;i++){
      newSteps[i]=steps[i];
    }
    this.form=steps[i].getForm();
    steps=newSteps;
  }

  /**
   * Navrat v derivaci k pozadovanemu kroku. Vsechny kroky, pred ktere se 
   * derivace vrati, jsou zapomenuty.
   *
   * @param index Poradi kroku, do ktereho ma derivace zustat 
   * @throws IndexOutOfBoundsException Pokud je index mimo existujici pocet 
   *         kroku
   */  
  public void backToStep(int index) throws IndexOutOfBoundsException{
    Step newSteps[]=new Step[index];
    int i;
    if(index==steps.length){
      return;
    }
    for(i=0;i<index;i++){
      newSteps[i]=steps[i];
    }
    this.form=steps[i].getForm();
    steps=newSteps;
  }

  /**
   * Slouzi k ziskani aktualni vetne formy
   * 
   * @return Aktualni vetna forma 
   */  
  public SententialForm getForm(){
    return this.form;
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych na aktualni vetnou 
   * formu.
   * 
   * @param left Pokud je nastaveno na true, pouziva leftmost nebo levou 
   *        derivaci podle definice konkretni gramatiky
   * @return Pole aplikovatelnych pravidel
   */  
  public Production[] usableProductions(boolean left){
    Step s;
    if(this.steps.length==0){
      s=null;
    }
    else{
      s=steps[steps.length-1];
    }
    if(left){
      return grammar.leftUsableProductions(form, s);
    }
    else{
      return grammar.usableProductions(form, s);
    }
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych ve smyslu testovani 
   * vyskytu na aktualni vetnou formu.
   * 
   * @param left Pokud je nastaveno na true, pouziva leftmost nebo levou 
   *        derivaci podle definice konkretni gramatiky
   * @return Pole pravidel aplikovatelnych ve smyslu testovani vyskytu
   */  
  public Production[] acUsableProductions(boolean left){
    Step s;
    if(this.steps.length==0){
      s=null;
    }
    else{
      s=steps[steps.length-1];
    }
    if(left){
      return grammar.leftAcUsableProductions(form, s);
    }
    else{
      return grammar.acUsableProductions(form, s);
    }
  }

  /**
   * Vytvori pole retezcu. V kazdem retezci je jeden krok ve tvaru:
   * <pre>vetna forma =&gt; pravidlo<pre> nebo
   * <pre>vetna forma =&gt; ac pravidlo<pre>
   * Retezec popisujici pravidlo je generovan objektem typu Production a muze se
   * lisit podle konkretni podtridy.
   * 
   * @return Pole retezcu urcene pro seznam kroku derivace 
   */  
  public String[] stepsToList(){
    String listStr[]=new String[steps.length+1];
    int i;
    for(i=0; i<steps.length; i++){
      StringBuffer sb=new StringBuffer();
      sb.append(steps[i].getForm());
      sb.append("   =>");
      if(steps[i].isAc()){
        sb.append("ac ");
      }
      sb.append(steps[i].getProduction().toStepsString());
      listStr[i]=sb.toString();
    }
    listStr[i]=form.toString();
    return listStr;
  }

  /**
   * Aplikuje pravidlo na prvni vyskyt leve strany ve vetne forme.
   * 
   * @param p Aplikovane pravidlo 
   */  
  public void applyProduction(Production p)
    throws IndexOutOfBoundsException{
    Step step=new Step(this.form, p, false);
    this.form=p.derivate(this.form);
    Step newSteps[]=new Step[this.steps.length+1];
    int i;
    for(i=0; i<steps.length; i++){
      newSteps[i]=steps[i];
    }
    newSteps[i]=step;
    steps=newSteps;
  }

  /**
   * Aplikuje pravidlo ve smyslu testovani vyskytu. Vetna forma se nezmeni a 
   * pribude dalsi krok derivace.
   * 
   * @param p Aplikovane pravidlo 
   */  
  public void applyAcProduction(Production p)
    throws IndexOutOfBoundsException{
    Step step=new Step(this.form, p, true);
    Step newSteps[]=new Step[this.steps.length+1];
    int i;
    for(i=0; i<steps.length; i++){
      newSteps[i]=steps[i];
    }
    newSteps[i]=step;
    steps=newSteps;
  }

  /**
   * Aplikuje pravidlo na urceny vyskyt leve strany ve vetne forme.
   * 
   * @param p Aplikovane pravidlo 
   * @param index Vyskyt leve strany, na ktery se ma pravidlo pouzit. 
   *        Pocita se od 1.
   */  
  public void applyProduction(Production p, int index)
    throws IndexOutOfBoundsException{

    SententialForm sf;
    Step step=new Step(this.form, p, false);
    this.form=p.derivate(this.form, index);
    Step newSteps[]=new Step[this.steps.length+1];
    int i;
    for(i=0; i<steps.length; i++){
      newSteps[i]=steps[i];
    }
    newSteps[i]=step;
    steps=newSteps;
  }

  /**
   * Slouzi ke zjisteni, jestli jiz bylo vygenerovano terminalni slovo.
   * 
   * @return Vraci true, pokud se ve vetne forme nevyskytuji neterminaly. Jinak
   *         vraci false.
   */  
  public boolean isTerminalWord(){
    return this.form.isTerminalWord();
  }

  /**
   * Slouzi ke zjisteni, kolik kroku derivace jiz probehlo
   * 
   * @return Pocet kroku derivace
   */  
  public int stepsCount(){
    return this.steps.length;
  }
}
