package grammars;
import java.util.*;

/**
 * Trida reprezentujici klasickou obecnou gramatiku. Obsahuje mnoziny 
 * terminalu, neterminalu, pravidel a pocatecni neterminal.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class Grammar {
  /**Jmeno terminalu, ktery reprezentuje prazdne slovo*/
  public static final String lambda = "la";

  /**Pole terminalu gramatiky*/
  protected Terminal[] terminals=new Terminal[0];
  /**Pole neterminalu gramatiky*/
  protected Nonterminal[] nonterminals=new Nonterminal[0];
  /**Pole pravidel gramatiky*/
  protected Production[] productions=new Production[0];
  /**Pocatecni neterminal*/
  protected Nonterminal start=null;
  /**Prazdne slovo*/
  protected Terminal lambdaTerm=new Terminal(lambda);

  /**
   * Konstruktor, ktery vytvori gramatiku bez pravidel, terminalu i neterminalu
   */
  public Grammar(){
  }

  /**
   * Konstruktor, ktery vytvori gramatiku bez terminalu i neterminalu. 
   * Pravidla jsou urcena.
   *
   * @param p Pole pravidel gramatiky
   */
  public Grammar(Production[] p){
    this.productions=p;
  }

  /**
   * Vytvori gramatiku se vsemi prvky 
   *
   * @param p Pole pravidel gramatiky
   * @param t Pole terminalu
   * @param n Pole neterminalu
   * @param s Pocatecni neterminal
   */
  public Grammar(Production[] p, Terminal[] t, Nonterminal[] n,
      Nonterminal s){
    this.productions=p;
    this.terminals=t;
    this.nonterminals=n;
    this.start=s;
  }
  
  /**
   * Vytvori kopii gramatiky.
   *
   * @return Kopie gramatiky
   */
  public Grammar copy(){
    Production p[] = new Production[this.productions.length];
    for(int i=0; i<productions.length; i++){
      p[i]=productions[i];
    }
    return new Grammar(p, terminals, nonterminals, start);
  }

  /**
   * Slouzi k pridani terminalu do gramatiky. Pokud jiz existuje terminal s 
   * danym jmenem, vrati jej. Jinak vytvori novy a vrati ho. Neterminal lambda
   * do mnoziny symbolu nepridava.
   *
   * @param name Retezec jmena terminalu
   * @return Terminal pozadovaneho jmena
   */
  public Terminal addTerminal(String name){
    int i;
    if(name.equals(lambda)){
      return lambdaTerm;
    }
    for(i=0; i<terminals.length; i++){
      if(terminals[i].hasName(name)){
        return terminals[i];
      }
    }
    Terminal newTerm[] = new Terminal[terminals.length+1];
    for(i=0; i<terminals.length; i++){
      newTerm[i] = terminals[i];
    }
    newTerm[i] = new Terminal(name);
    terminals=newTerm;
    return terminals[i];
  }

  /**
   * Nastavi pocatecni neterminal na pozadovanou hodnotu
   * 
   * @param s Novy pocatecni neterminal
   */
  public void setStart(Nonterminal s){
    this.start=s;
  }

 /**
  * Slouzi k ziskani pocatecniho neterminalu
  *
  * @return Pocatecni neterminal
  */
  public Nonterminal getStart(){
    return this.start;
  }

  /**
   * Slouzi k pridani neterminalu do gramatiky. Pokud jiz existuje neterminal s 
   * danym jmenem, vrati jej. Jinak vytvori novy a vrati ho.
   *
   * @param name Retezec jmena neterminalu
   * @return Neterminal pozadovaneho jmena
   */
  public Nonterminal addNonterminal(String name){
    int i;
    for(i=0; i<nonterminals.length; i++){
      if(nonterminals[i].hasName(name)){
        return nonterminals[i];
      }
    }
    Nonterminal newTerm[] =
      new Nonterminal[nonterminals.length+1];
    for(i=0; i<nonterminals.length; i++){
      newTerm[i] = nonterminals[i];
    }
    newTerm[i] = new Nonterminal(name);
    nonterminals=newTerm;
    return nonterminals[i];
  }

  /**
   * Slouzi k pridani libovolneho symbolu do gramatiky. Je-li prvni znak jmena
   * velke pismeno je pridan jako neterminal, je-li prvni male tak jako 
   * terminal.
   * 
   * @param name Jmeno symbolu
   * @return Pridany symbol. Pokud prvni znak neni pismeno, vraci null
   */
  public Symbol addSymbol(String name){
    if(Character.isUpperCase(name.charAt(0))){
      return this.addNonterminal(name);
    }
    if(Character.isLowerCase(name.charAt(0))){
      return this.addTerminal(name);
    }
    return null;
  }

  /**
   * Prida jedno pravidlo do pole pravidel 
   *
   * @param p Pridavane pravidlo
   */
  public void addProduction(Production p){
    Production p2[]=new Production[this.productions.length+1];
    int i;

    for(i=0;i<productions.length;i++){
      p2[i]=productions[i];
    }
    p2[i]=p;
    productions=p2;
    changeUseCount(p,1);
  }

  /**
   * Odstrani jedno pravidlo z pole pravidel.
   *
   * @param index Poradi pravidla v poli pravidel. Odpovida poradi, v jakem
   * byla pravidla pridavana. Pocita se od 0.
   */
  public void removeProduction(int index){
    if(index>=0 && index<productions.length){
      Production p2[]=new Production[this.productions.length-1];
      int i,j;
      j=0;
      for(i=0;i<productions.length;i++){
        if(i!=index){
          p2[j++]=productions[i];
        }
        else{
          changeUseCount(productions[index],-1);
        }
      }
      productions=p2;
      noUsedSymbols();
    }
  }

  /**
   * U kazdeho terminalniho i neterminalniho symbolu si gramatika pamatuje,
   * kolikrat je obsazen v pravidlech. Pri odstraneni pravidla s poslednim
   * vyskytem symbolu je tento symbol odstranen z mnoziny terminalu nebo
   * neterminalu. Tato funkce zmeni vsem symbolum v pravidle pocet vyskytu
   * prictenim pozadovane hodnoty.
   *
   * @param p Pravidlo, jehoz symbolum se meni pocet vyskytu
   * @param change Hodnota prictena k poctu vyskytu
   */
  private void changeUseCount(Production p, int change){
    SententialForm sf;
    sf=p.getLeft();
    for(int i=0; i<sf.length(); i++){
      sf.symbolAt(i).changeUseCount(change);
    }
    sf=p.getRight();
    for(int i=0; i<sf.length(); i++){
      sf.symbolAt(i).changeUseCount(change);
    }
  }

  /**
   * Odstrani z mnozin neterminalu a terminalu ty symboly, ktere se nevyskytuji
   * v zadnem pravidle
   *
   */
  private void noUsedSymbols(){
    Vector v=new Vector();
    for(int i=0; i<terminals.length; i++){
      if(terminals[i].getUseCount() > 0){
        v.add(terminals[i]);
      }
    }
    terminals=(Terminal[]) v.toArray(new Terminal[v.size()]);
    v=new Vector();
    for(int i=0; i<nonterminals.length; i++){
      if(nonterminals[i].getUseCount() >0){
        v.add(nonterminals[i]);
      }
    }
    nonterminals=(Nonterminal[]) v.toArray(new Nonterminal[v.size()]);
  }

  /**
   * Slouzi k ziskani vsech pravidel gramatiky
   *
   * @return Pole pravidel gramatiky
   */
  public Production[] getProductions(){
    return this.productions;
  }

  /**
   * Slouzi k ziskani vsech terminalu gramatiky
   *
   * @return Pole terminalu gramatiky
   */
  public Terminal[] getTerminals(){
    return terminals;
  }

  /**
   * Slouzi k ziskani vsech neterminalu gramatiky
   *
   * @return Pole neterminalu gramatiky
   */
  public Nonterminal[] getNonterminals(){
    return nonterminals;
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych na urcenou vetnou
   * formu. Pro pripadne podtridy, ktere by mohly pro urceni aplikovatelnych
   * pravidel potrebovat znat pravidlo pouzite v minulem kroku derivace,
   * je predchozi krok parametrem funkce.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] usableProductions(SententialForm sent,
      Step lastStep){
    int i;
    Vector v = new Vector();
    for(i=0; i<productions.length; i++){
      if(SententialForm.isSubword(sent,productions[i].getLeft())){
        v.add(this.productions[i]);
      }
    }
    return (Production[])v.toArray(new Production[v.size()]);
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych na urcenou vetnou
   * formu. Pri vyberu je vybrano pravidlo nebo pravidla, jejichz leva strana
   * je ve vetne forme nejvice vlevo. Pro pripadne podtridy, ktere by mohly 
   * pro urceni aplikovatelnych pravidel potrebovat znat pravidlo pouzite v 
   * minulem kroku derivace, je predchozi krok parametrem funkce.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] leftUsableProductions(SententialForm sent,
      Step lastStep){
    int i;
    Vector v = new Vector();
    int left, pos;
    left=sent.length()+1;
    for(i=0; i<productions.length; i++){
      if((pos=sent.indexOf(productions[i].getLeft()))!=-1){
        if(pos==left){
          v.add(this.productions[i]);
        }
        if(pos<left){
          v.clear();
          v.add(this.productions[i]);
          left=pos;
        }
      }
    }
    return (Production[])v.toArray(new Production[v.size()]);
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych ve smyslu testovani
   * vyskytu na urcenou vetnou formu. Pro pripadne podtridy, ktere by mohly 
   * pro urceni aplikovatelnych pravidel potrebovat znat pravidlo pouzite v 
   * minulem kroku derivace, je predchozi krok parametrem funkce.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] acUsableProductions(SententialForm sent,
      Step lastStep){
    return new PrgProduction[0];
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych ve smyslu testovani
   * vyskytu na urcenou vetnou formu pri soucasnem uvazovani leve nebo leftmost
   * derivace. Pro pripadne podtridy, ktere by mohly 
   * pro urceni aplikovatelnych pravidel potrebovat znat pravidlo pouzite v 
   * minulem kroku derivace, je predchozi krok parametrem funkce.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] leftAcUsableProductions(SententialForm sent,
      Step lastStep){
    return new PrgProduction[0];
  }
}
