package grammars;
import java.io.*;
import java.util.*;


/**
 * Trida slouzici pro ulozeni programovane gramatiky do souboru a jeji opetovne
 * nacteni.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class PrgGrammarFile {
  /**
   * Znaky, ktere se nesmi vyskytovat v terminalech, neterminalech a oznacenich
   * pravidel. Slouzi totiz k jejich vzajemnemu oddeleni.
   */
  public static final String delimiters= " \n\r\t\f,:;[]{}()-=<>";

  /**Soubor, do ktereho se bude ukladat, nebo, ze ktereho se bude cist*/
  protected File file=null;

  /**
   * Pomocny konstruktor bez parametru
   */
  public PrgGrammarFile() {
  }
 
  /** 
   * Vytvori tridu s urcenym souborem s gramatikou
   *
   * @param f Soubor s gramatikou
   */
  public PrgGrammarFile(File f) {
    this.file = f;
  }

  /**
   * Ulozi gramatiku do souboru
   *
   * @param g Ukladana gramatika
   * @throws java.io.IOException Pri chybe v zapise do souboru
   */
  public void writeGrammar(Grammar g)
                             throws java.io.IOException{
    PrintStream out = new PrintStream(new FileOutputStream(file));
    ProgrammedGrammar gram=(ProgrammedGrammar) g;
    Production prods[]=gram.getProductions();
    Symbol s=gram.getStart();
    out.println(s.toString());

    for(int i=0; i<prods.length; i++){
      out.println(prods[i].toListItem());
    }
  }

  /**
   * Nacte gramatiku ze souboru
   *
   * @return Nactena programovana gramatika
   * @throws java.io.FileNotFoundException Soubor nebyl nalezen
   * @throws java.text.ParseException Spatny format souboru. Vyjimka ma v 
   * retezci popis chyby a ve druhem parametru radek, na kterem chyba nastala
   * @throws java.io.IOException Chyba pri cteni ze souboru
   */
  public ProgrammedGrammar readGrammar() throws FileNotFoundException,
                                      java.text.ParseException,
                                      IOException{
    BufferedReader reader = new BufferedReader(new InputStreamReader(
      new FileInputStream(file)));
    String line=null;
    int lineNum=0, count;
    String s;
    ProgrammedGrammar gram = new ProgrammedGrammar();
    StringTokenizer token;

    while(true){  //Cte radky, dokud nanacte pocatecni neterminal
      line=reader.readLine();
      lineNum++;
      if(line==null){ //Konec souboru
        throw new java.text.ParseException("Ocekavan pocatecni neterminal",
          lineNum);
      }
      line=line.trim(); //Odstrani bile znaky
      if(line.startsWith("#")||line.length()==0){ //Odstrani komentare
        continue;
      }

      token=new StringTokenizer(line,delimiters);
      count = token.countTokens();
      if(count==0||count>1){  //Neni neterminal nebo jich je vice
        throw new java.text.ParseException("Ocekavan pocatecni neterminal",
          lineNum);
      }
      else{
        s=token.nextToken();
      }
      if(!Character.isUpperCase(s.charAt(0))){  
        throw new java.text.ParseException(
          "Pocatecni neterminal musi zacinat velkym pismenem",lineNum);
      }
      gram.setStart(gram.addNonterminal(s));  //Nastavi pocatecni neterminal
      break;
    }
    while(true){        //Cte pravidla az do konce souboru
      line=reader.readLine();
      lineNum++;
      if(line==null){   //Konec souboru
        break;
      }
      line=line.trim();
      if(line.startsWith("#")||line.length()==0){ //Komentare a prazdne radky
        continue;
      }
      parseProduction(line, lineNum, gram);  //Nacte z radku pravidlo
    }
    return gram;
  }

  /**
   * Rozdeli jeden radek na pravidlo a ulozi ho do gramatiky
   *
   * @throws java.text.ParseException Spatny format souboru. Vyjimka ma v 
   * retezci popis chyby a ve druhem parametru radek, na kterem chyba nastala
   * @param line Jeden radek souboru reprezentujici jedno pravidlo
   * @param lineNum Cislo radku souboru, ze ktereho byl retezec nacten
   * @param gram Programovana gramatika, do ktere se ma pravidlo ulozit
   */
  protected void parseProduction(String line, int lineNum, 
	    ProgrammedGrammar gram)
            throws java.text.ParseException{
    int count;
    int leftPos, rightPos;
    PrgProduction prod;
    StringTokenizer token;
    String labelStr;
    String left;
    String[] right;
    String[] succ;
    String[] fail;

    leftPos=0;
    rightPos=line.indexOf(":");        //najde konec oznaceni
    if(rightPos==-1){                  //nenalezena :
      throw new java.text.ParseException(
        "Za oznacenim pravilda ocekavan znak ':'",lineNum);
    }
    labelStr = line.substring(leftPos,rightPos); //oddeli jen oznaceni pravidla
    token=new StringTokenizer(labelStr, delimiters);
    count = token.countTokens();
    if(count==0||token.countTokens()>1){  
                                   //v oznaceni pravidla jsou nepovelene znaky
      throw new java.text.ParseException(
        "Ocekavano oznaceni pravidla",lineNum);
    }
    if(gram.existLabel(labelStr)){      //oznaceni jiz bylo uzito
      throw new java.text.ParseException(
        "Oznaceni pravidla jiz bylo pouzito",lineNum);
    }


    leftPos=rightPos+1;   //pokracuje ve zbytku retezce
    rightPos=line.indexOf("->",leftPos);  //hleda oddeleni prave strany
    if(rightPos==-1){           //sipka nenalezena
      throw new java.text.ParseException(
        "Za levou stranou pravilda ocekavan retezec \"->\"",lineNum);
    }
    left = line.substring(leftPos,rightPos);  //jen prava strana pravidla
    token=new StringTokenizer(left, delimiters);
    count = token.countTokens();
    if(count==0||token.countTokens()>1){  //nema pozadovany tvar
      throw new java.text.ParseException(
        "Na leve strane pravilda ocekavan jeden neterminal",lineNum);
    }
    left=token.nextToken(); 
    if(!Character.isUpperCase(left.charAt(0))){ 
      throw new java.text.ParseException(  //Nejde o neterminal
        "Neterminal na leve strane pravidla nezacina velkym pismenem",lineNum);
    }


    leftPos=rightPos+2;  //pokracuje ve zbytku retezce
    rightPos=line.indexOf("{",leftPos);   //hleda zacatek pole uspechu
    if(rightPos==-1){
      throw new java.text.ParseException(
        "Za pravou stranou pravilda ocekavan znak '{'",lineNum);
    }
    token=new StringTokenizer(line.substring(leftPos,rightPos),
      delimiters);
    count = token.countTokens();
    if(count==0){
      throw new java.text.ParseException(
        "Na prave strane pravilda ocekavan alespon jeden symbol",lineNum);
    }
    right=new String[count];
    for(int i=0; token.hasMoreElements(); i++){
      right[i]=token.nextToken();     //Do pole si rozdeli jednotlive symboly
    }

    Symbol r[]=new Symbol[right.length];
    for(int i=0; i<right.length;i++){  //Z retezcu vytvori symboly a prida je
      r[i]=gram.addSymbol(right[i]);   //do gramatiky
      if(r[i]==null){
        throw new java.text.ParseException(
          "Symbol musi zacinat pismenem",lineNum);
      }
    }

    Symbol l=gram.addSymbol(left);  //Prida i levou stranu
    if(l==null){
        throw new java.text.ParseException(
          "Symbol musi zacinat pismenem",lineNum);
    }


    leftPos=rightPos+1;  //Pokracuje zbytkem retezce
    rightPos=line.indexOf("}",leftPos); //Hleda konec pole uspechu
    if(rightPos==-1){
      throw new java.text.ParseException(
        "Pole uspechu musi koncit '}'",lineNum);
    }
    token=new StringTokenizer(line.substring(leftPos,rightPos),delimiters);
    count = token.countTokens();
    succ=new String[count];
    for(int i=0; token.hasMoreElements(); i++){
      succ[i]=token.nextToken();  //Rozdeli pole uspechu na jednotlive oznaceni
    }


    leftPos=rightPos+1;  //pokracuje zbytkem retezce
    rightPos=line.indexOf("{",leftPos);  //Hleda zacatek pole neuspechu
    if(rightPos==-1){
      throw new java.text.ParseException(
        "Pole neuspechu musi zacinat '{'",lineNum);
    }

    leftPos=rightPos+1;  //Pokracuje zbytkem retezce
    rightPos=line.indexOf("}",leftPos);  //Hleda konec pole neuspechu
    if(rightPos==-1){
      throw new java.text.ParseException(
        "Pole neuspechu musi koncit '}'",lineNum);
    }
    token=new StringTokenizer(line.substring(leftPos,rightPos),delimiters);
    count = token.countTokens();
    fail=new String[count];
    for(int i=0; token.hasMoreElements(); i++){
      fail[i]=token.nextToken();  //Rozdeli na jednotliva oznaceni pravidel
    }

    prod=new PrgProduction(labelStr, new SententialForm(l),
      new SententialForm(r));    //Vytvori nactene pravidlo
    prod.setFailLabels(fail);   //Nastavi pravidlu oznaceni pravidel v poli
                                //neuspechu
    prod.setSuccesLabels(succ); //Nastavi pravidlu oznaceni pravidel v poli 
                                //uspechu
    gram.addProduction(prod);   //Prida pravidlo do gramatiky
  }
}
