package grammars;


/**
 * Reprezentace pravidla programovane gramatiky. Rozsiruje klasicke pravidlo 
 * popsane tridou Production o pole uspechu a neuspechu a oznaceni pravidla
 *
 * @author Martin Kot
 * @version 1.0
 */
public class PrgProduction extends Production {
  /**Pole uspechu reprezentovane polem pravidel*/
  protected PrgProduction[] succes=new PrgProduction[0];
  /**Pole neuspechu reprezentovane polem pravidel*/
  protected PrgProduction[] fail=new PrgProduction[0];
  /**Oznaceni pravidla*/
  protected String label;
  /**Oznaceni pravidel patricich do pole uspechu tohoto pravidla*/
  protected String[] succesLabels;
  /**Oznaceni pravidel patricich do pole neuspechu tohoto pravidla*/
  protected String[] failLabels;

  /**
   * Pomocny konstruktor. Vytvori pravidlo bez vsech prvku.
   */
  public PrgProduction() {
    super();
    this.succes=null;
    this.fail=null;
    this.label=null;
  }
  
  /**
   * Konstruktor pravidla s oznacenim, levou a pravou stranou. Pole uspechu
   * i neuspechu jsou prazdna.
   *
   * @param l Oznaceni pravidla
   * @param sf1 Leva strana pravidla
   * @param sf2 Prava strana pravidla
   */
  public PrgProduction(String l, SententialForm sf1, SententialForm sf2){
    this.left=sf1;
    this.right=sf2;
    this.succes=null;
    this.fail=null;
    this.label=l;
  }

  /**
   * Konstruktor pravidla s levou a pravou stranou. Pole uspechu
   * i neuspechu jsou prazdna.
   *
   * @param sf1 Leva strana pravidla
   * @param sf2 Prava strana pravidla
   */
  public PrgProduction(SententialForm sf1, SententialForm sf2){
    this.left=sf1;
    this.right=sf2;
    this.succes=null;
    this.fail=null;
    this.label=null;
  }
  
  /**
   * Nastavi pole uspechu
   *
   * @param prods Pravidla noveho pole uspechu
   */
  public void setSucces(PrgProduction[] prods){
    this.succes=prods;
  }

  /**
   * Nastavi pole neuspechu
   *
   * @param prods Pravidla noveho pole neuspechu
   */
  public void setFail(PrgProduction[] prods){
    this.fail=prods;
  }

  /**
   * Slouzi k ziskani pravidel patricich do pole uspechu
   *
   * @return Pole pravidel patricich do pole uspechu
   */
  public PrgProduction[] getSucces(){
    return this.succes;
  }

  /**
   * Slouzi k ziskani pravidel patricich do pole neuspechu
   *
   * @return Pole pravidel patricich do pole neuspechu
   */
  public PrgProduction[] getFail(){
    return this.fail;
  }

  /**
   * Slouzi k ziskani oznaceni pravidel patricich do pole uspechu
   *
   * @return Pole oznaceni pravidel patricich do pole uspechu
   */
  public String[] getSuccesLabels(){
    return this.succesLabels;
  }

  /**
   * Slouzi k ziskani oznaceni pravidel patricich do pole neuspechu
   *
   * @return Pole oznaceni pravidel patricich do pole neuspechu
   */
  public String[] getFailLabels(){
    return this.failLabels;
  }

  /**
   * Slouzi k ziskani oznaceni pravidla
   *
   * @return Oznaceni pravidla
   */
  public String getLabel(){
    return this.label;
  }

  /**
   * Zmeni oznaceni pravidel patricich do pole uspechu
   *
   * @param labs Pole oznaceni pravidel patricich do pole uspechu
   */
  public void setSuccesLabels(String[] labs){
    this.succesLabels = labs;
  }

  /**
   * Zmeni oznaceni pravidel patricich do pole neuspechu
   *
   * @param labs Pole oznaceni pravidel patricich do pole neuspechu
   */
  public void setFailLabels(String[] labs){
    this.failLabels = labs;
  }

  /**
   * Slouzi k zjisteni, jestli ma pravidlo pozadovane oznaceni
   * 
   * @param l Pozadovane oznaceni
   * @return true, pokud je oznaceni pravidla shodne s parametrem l, jinak 
   * false
   */
  public boolean hasLabel(String l){
    if(this.label.equals(l)){
      return true;
    }
    else{
      return false;
    }
  }
  
  /**
   * Vytvori retezec z oznaceni pravidel patricich do pole uspechu. Jednotliva 
   * oznaceni oddeli carkou a mezerou. Za posledni jiz carku neda.
   *
   * @return Retezec oznaceni pravidel z pole uspechu
   */
  public String succesToString(){
    StringBuffer sb = new StringBuffer();
    for(int i=0; i< succesLabels.length; i++){
      if(i!=0){
        sb.append(", ");
      }
      sb.append(succesLabels[i]);
    }
    return sb.toString();
  }

  /**
   * Vytvori retezec z oznaceni pravidel patricich do pole neuspechu. 
   * Jednotliva oznaceni oddeli carkou a mezerou. Za posledni jiz carku neda.
   *
   * @return Retezec oznaceni pravidel z pole neuspechu
   */
  public String failToString(){
    StringBuffer sb = new StringBuffer();
    for(int i=0; i< failLabels.length; i++){
      if(i!=0){
        sb.append(", ");
      }
      sb.append(failLabels[i]);
    }
    return sb.toString();
  }

  /**
   * Vytvori retezec reprezentujici pravidlo v seznamu pouzitelnych pravidel.
   * Tvar je <pre> [oznaceni] pravidlo {pole uspechu} {pole neuspechu} <pre>
   * kde tvar pravidla je dan funkci toListItem() tridy Production a pole 
   * uspechu a neuspechu jsou carkami oddelena oznaceni pravidel
   *
   * @return Retezec reprezentujici pravidlo
   */
  public String toListItem(){
    StringBuffer sb = new StringBuffer();
    sb.append(this.label.toString());
    sb.append(": ");
    sb.append(super.toString());
    sb.append(" {");
    sb.append(succesToString());
    sb.append("} {");
    sb.append(failToString());
    sb.append("}");
    return sb.toString();
  }

  /**
   * Vytvori retezec reprezentujici pravidlo v seznamu kroku derivace.
   * Tvar je <pre> [oznaceni]: (pravidlo) <pre>
   * kde tvar pravidla je dan funkci toString() tridy Production
   *
   * @return Retezec reprezentujici pravidlo
   */
  public String toStepsString(){
    StringBuffer sb=new StringBuffer();
    sb.append("[");
    sb.append(this.label);
    sb.append("]: (");
    sb.append(toString());
    sb.append(")");
    return sb.toString();
  }
}
