package grammars;

/**
 * Trida reprezentujici pravidlo klasicke obecne gramatiky
 * @author Martin Kot
 * @version 1.0
 */
public class Production {
  /**Leva strana pravidla*/
  protected SententialForm left;
  /**Prava strana pravidla*/
  protected SententialForm right;

  /**
   * Pomocny konstruktor. Vytvori pravidlo s prazdou pravou i levou stranou
   */
  public Production() {
    this.left=new SententialForm(new Symbol[0]);
    this.right=new SententialForm(new Symbol[0]);
  }

  /**
   * Vytvori pravidlo s pozadovanou levou i pravou stranou
   *
   * @param l Leva strana pravidla
   * @param p Prava strana pravidla
   */
  public Production(SententialForm l, SententialForm r) {
    this.left=l;
    this.right=r;
  }

  /**
   * Slouzi k ziskani leve strany pravidla
   *
   * @return Leva strana pravidla
   */
  public SententialForm getLeft(){
    return this.left;
  }
 
  /**
   * Slouzi k ziskani prave strany pravidla
   *
   * @return Prava strana pravidla
   */
  public SententialForm getRight(){
    return this.right;
  }

  /**
   * Slouzi k nastaveni leve strany pravidla
   *
   * @param sf Nova leva strana pravidla
   */
  public void setLeft(SententialForm sf){
    this.left=sf;
  }

  /**
   * Slouzi k nastaveni prave strany pravidla
   *
   * @param sf Nova prava strana pravidla
   */
  public void getRight(SententialForm sf){
    this.right=sf;
  }
 
  /**
   * Provede prepsani nejlevejsiho vyskytu leve strany ve vetne forme
   * 
   * @return Nova vetna forma. Je ziskana z puvodni nahrazenim leve strany
   * pravidla pravou stranou.
   * @param oldSF Puvodni vetna forma, na kterou se pravidlo aplikuje
   */
  SententialForm derivate(SententialForm oldSF){
    int position;
    SententialForm newSF;
    position=oldSF.indexOf(this.left);
    if(position<0){
      return new SententialForm(oldSF); //Leva strana se nevyskytuje
    }
    else{
      if(right.length()==1){
        if(right.symbolAt(0).hasName(Grammar.lambda)){ //Vypousteci pravidlo
          SententialForm sf=new SententialForm(oldSF.subword(0,position),
            oldSF.subword(position+left.length()));  //Odstrani levou stranu
          if(sf.length()==0){  //Vetnou formu delky nula nahradi lambda
            return new SententialForm(new Symbol(Grammar.lambda));
          }
          else{
            return sf;
          }
        }
      }

      return new SententialForm(oldSF.subword(0,position), //Nahradi levou
        right, oldSF.subword(position+left.length()));  //stranu za pravou
    }
  }

  /**
   * Provede prepsani pozadovaneho vyskytu leve strany ve vetne forme
   * 
   * @return Nova vetna forma. Je ziskana z puvodni nahrazenim leve strany
   * pravidla pravou stranou.
   * @param oldSF Puvodni vetna forma, na kterou se pravidlo aplikuje
   * @param index Poradi vyskytu leve strany, ktery ma byt prepsan. Pocita
   * se od 1.
   * @throws IndexOutOfBoundsException Pokud je pozadovan neexistujici vyskyt
   */
  public SententialForm derivate(SententialForm oldSF, int index)
    throws IndexOutOfBoundsException{

    int position[];
    SententialForm newSF;
    position=oldSF.allIndexOf(this.left);
    if(position.length==0){
      return new SententialForm(oldSF);
    }
    else if(position.length<index){
      throw new IndexOutOfBoundsException();
    }
    else{
      if(right.length()==1){
        if(right.symbolAt(0).hasName(Grammar.lambda)){ //Vypousteci pravidlo
                               //Odstrani levou stranu
          SententialForm sf=new SententialForm(
            oldSF.subword(0,position[index-1]),
            oldSF.subword(position[index-1]+left.length()));  
          if(sf.length()==0){  //Vetnou formu delky nula nahradi lambda
            return new SententialForm(new Symbol(Grammar.lambda));
          }
          else{
            return sf;
          }
        }
      }
      return new SententialForm(oldSF.subword(0,position[index-1]),
        right, oldSF.subword(position[index-1]+left.length()));
    }
  }

  /**
   * Prevede pravidlo na retezec ve tvaru 
   * <pre> leva strana -&gt; prava strana</pre>
   *
   * @return Retezec reprezentujici pravidlo
   */
  public String toString(){
    StringBuffer sb;
    sb = new StringBuffer();
    if(left!=null){
      sb.append(left.toString());
    }
    sb.append(" -> ");
    if(right!=null){
      sb.append(right.toString());
    }
    return sb.toString();
  }

  /**
   * Prevede pravidlo na retezec pro seznam pouzitelnych pravidel. Tvar je 
   * stejny, jako generovany funkci toString()
   *
   * @return Retezec reprezentujici pravidlo
   */
  public String toListItem(){
    return toString();
  }

  /**
   * Prevede pravidlo na retezec pro seznam kroku derivace. Tvar je 
   * stejny, jako generovany funkci toString() jen uzavreny do zavorek
   *
   * @return Retezec reprezentujici pravidlo
   */
  public String toStepsString(){
    StringBuffer sb=new StringBuffer();
    sb.append("(");
    sb.append(toString());
    sb.append(")");
    return sb.toString();
  }
}
