package grammars;
import java.util.*;

/**
 * Trida reprezentujici programovanou gramatiku. Proti klasicke gramatice 
 * pridava praci s oznacenimi pravidel a meni zpusob hledani pouzitelnych
 * pravidel.
 *
 * @author Martin Kot
 * @version 1.0
 */
public class ProgrammedGrammar extends Grammar{

  /**
   * Pomocny konstruktor. Vytvori gramatiku bez pravidel i symbolu.
   */
  public ProgrammedGrammar() {
    super();
  }

  /**
   * Vytvori gramatiku s urcenymi pravidly
   *
   * @param p Pravidla nove gramatiky
   */
  public ProgrammedGrammar(PrgProduction p[]) {
    super(p);
  }

  /**
   * Vytvori gramatiku s urcenymi pravidly i symboly
   *
   * @param p Pravidla nove gramatiky
   * @param t Mnozina terminalu
   * @param n Mnozina neterminalu
   * @param s Pocatecni neterminal
   */
  public ProgrammedGrammar(Production[] p, Terminal[] t, Nonterminal[] n,
      Nonterminal s){
    super(p, t, n, s);
  }

  /**
   * Vytvori kopii gramatiky. Pravidla, terminaly a neterminaly netvori znovu,
   * jen zkopiruje odkazy na ne.
   *
   * @return Kopie gramatiky
   */
  public Grammar copy(){
    Production p[] = new Production[this.productions.length];
    for(int i=0; i<productions.length; i++){
      p[i]=productions[i];
    }
    return new ProgrammedGrammar(p, terminals, nonterminals, start);
  }

  /**
   * Slouzi ke zjisteni, jestli gramatika jiz obsahuje pravidlo s danym 
   * oznacenim.
   *
   * @param str Oznaceni hledane v pravidlech
   * @return true, pokud jiz oznaceni bylo pouzito. Jinak false.
   */
  public boolean existLabel(String str){
    for(int i=0; i<productions.length; i++){
      if(str.equals(((PrgProduction)this.productions[i]).getLabel())){
        return true;
      }
    }
    return false;
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych na urcenou vetnou
   * formu. Podle predchoziho kroku derivace urci pravidla, ze kterych se 
   * vybiraji pouzitelna.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] usableProductions(SententialForm sent,
      Step lastStep){
    int i;
    Vector v = new Vector();
    Production[] subset;
    PrgProduction lastProd;
    if(lastStep!=null){
      lastProd=(PrgProduction)lastStep.getProduction();
      if(lastStep.isAc()){   //Minuly krok byl ve smyslu testovani vyskytu
        subset=lastProd.getFail();  //Vybira z pole neuspechu
      }
      else{
        subset=lastProd.getSucces();  //Vybira z pole uspechu
      }
      if(subset==null){       //Pole bylo prazdne, tak nemusi ani pokracovat
        return new PrgProduction[0];
      }
    }
    else{                         //V prvnim kroku je pouzitelne vse
      subset=this.productions;
    }
    for(i=0; i<subset.length; i++){ 
      if(SententialForm.isSubword(sent,subset[i].getLeft())){
        v.add(subset[i]);  //Prida pravidla, jejichz leva strana se nachazi
      }                    //ve vetne forme
    }
    return (PrgProduction[])v.toArray(new PrgProduction[v.size()]);
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych na urcenou vetnou
   * formu. Pri vyberu je vybrano pravidlo nebo pravidla, jejichz leva strana
   * je ve vetne forme nejvice vlevo. Podle predchoziho kroku derivace urci 
   * pravidla, ze kterych se vybiraji pouzitelna.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] leftUsableProductions(SententialForm sent,
      Step lastStep){
    int i;
    Vector v = new Vector();
    Production[] subset;
    PrgProduction lastProd;
    int left, pos;
    if(lastStep!=null){
      lastProd=(PrgProduction)lastStep.getProduction();
      if(lastStep.isAc()){
        subset=lastProd.getFail(); //Vybira z pole neuspechu
      }
      else{
        subset=lastProd.getSucces(); //Vybira z pole uspechu
      }
      if(subset==null){   //Pole bylo prazdne, tak nemusi ani pokracovat
        return new PrgProduction[0];
      }
    }
    else{                         //V prvnim kroku je pouzitelne vse
      subset=this.productions;
    }
    left=sent.length()+1;
    for(i=0; i<subset.length; i++){
      if((pos=sent.indexOf(subset[i].getLeft()))!=-1){
        if(pos==left){
          v.add(subset[i]);  //Prida pravidla, jehoz leva strana se nachazi
        }                    //ve vetne forme na stejne pozici, jako zatim
                             //nejlevejsi
        if(pos<left){
          v.clear();         //Zapomene pravidla, ktera by se aplikovala
                             //vice vpravo
          v.add(subset[i]);  //Prida pravidla, jehoz leva strana se nachazi
                             //ve vetne forme vice vlevo nez zatim
                             //nejlevejsi
          left=pos;          //Zapamatuje si pozici leve strany ve vetne forme
        }
      }
    }
    return (PrgProduction[])v.toArray(new PrgProduction[v.size()]);
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych ve smyslu testovani
   * vyskytu na urcenou vetnou formu. Podle predchoziho kroku derivace urci 
   * pravidla, ze kterych se vybiraji pouzitelna.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] acUsableProductions(SententialForm sent,
      Step lastStep){
    int i;
    PrgProduction lastProd;
    Vector v = new Vector();
    Production[] subset;
    if(lastStep!=null){
      lastProd=(PrgProduction)lastStep.getProduction();
      if(lastStep.isAc()){
        subset=lastProd.getFail(); //Vybira z pole neuspechu
      }
      else{
        subset=lastProd.getSucces(); //Vybira z pole uspechu
      }
      if(subset==null){    //Pole bylo prazdne, tak nemusi ani pokracovat
        return new PrgProduction[0];
      }
    }
    else{                         //V prvnim kroku je pouzitelne vse
      subset=this.productions;
    }
    for(i=0; i<subset.length; i++){
      if(!SententialForm.isSubword(sent,subset[i].getLeft())){
        v.add(subset[i]);
      }
    }
    return (PrgProduction[])v.toArray(new PrgProduction[v.size()]);
  }

  /**
   * Slouzi k ziskani pravidel gramatiky aplikovatelnych ve smyslu testovani
   * vyskytu na urcenou vetnou formu pri soucasnem uvazovani leve nebo leftmost
   * derivace. Podle predchoziho kroku derivace urci pravidla, ze kterych se 
   * vybiraji pouzitelna. Pokud je nejake pravidlo pouzitelne klasicky, vraci
   * prazdne pole.
   *
   * @return Aplikovatelna pravidla
   * @param sent Vetna forma, na kterou se maji pravidla aplikovat
   * @param lastStep Predchozi krok derivace
   */
  public Production[] leftAcUsableProductions(SententialForm sent,
      Step lastStep){
    int i;
    PrgProduction lastProd;
    Vector v = new Vector();
    Production[] subset;
    if(lastStep!=null){
      lastProd=(PrgProduction)lastStep.getProduction();
      if(lastStep.isAc()){
        subset=lastProd.getFail(); //Vybira z pole neuspechu
      }
      else{
        subset=lastProd.getSucces(); //Vybira z pole uspechu
      }
      if(subset==null){     //Pole bylo prazdne, tak nemusi ani pokracovat
        return new PrgProduction[0];
      }
    }
    else{                         //V prvnim kroku je pouzitelne vse
      subset=this.productions;
    }
    for(i=0; i<subset.length; i++){
      if(!SententialForm.isSubword(sent,subset[i].getLeft())){
        v.add(subset[i]);        //Leva strana se nevyskytuje ve vetne forme
      }                          
      else{                      //Existuje pravidlo pouzitelne klasicky
        return new PrgProduction[0];
      }
    }
    return (PrgProduction[])v.toArray(new PrgProduction[v.size()]);
  }

  /**
   * Projde pravidla a vytvori v nich pole pravidel podle poli oznaceni pro 
   * pole uspechu a neuspechu.
   * 
   * @return Oznaceni, ktere nema zadne pravidlo, ale vyskytuje se v poli 
   * uspechu nebo neuspechu nejakeho pravidla. Pokud takove neexistuje, vraci
   * null a uspesne vytvori dve pole objektu typu PrgProduction
   * pravidla
   */
  public String labelToProds(){
    String labels[];
    PrgProduction prods[];
    int i,j,k;
    /*Prochazi vsechna pole uspechu*/
    for(i=0; i<this.productions.length; i++){
      labels = ((PrgProduction)productions[i]).getSuccesLabels();
      prods=new PrgProduction[labels.length];
      for(j=0; j<labels.length;j++){  //Prochazi jedno pole uspechu
        prods[j]=null;
        for(k=0;k<productions.length;k++){ //Hleda oznaceni mezi vsemi pravidly
          if(((PrgProduction)productions[k]).hasLabel(labels[j])){
            prods[j]=(PrgProduction)productions[k]; //Nalezene prida do pole
            break;                   //a pokracuje hledanim dalsiho
          }
        }
        if(prods[j]==null){  //Pozadovane oznaceni nema zadne pravidlo
          return labels[j];  //Vraci toto oznaceni
        }
      }
      ((PrgProduction)productions[i]).setSucces(prods); //Nastavi pole pravidel
    }

    /*Prochazi vsechna pole neuspechu*/
    for(i=0; i<this.productions.length; i++){
      labels = ((PrgProduction)productions[i]).getFailLabels();
      prods=new PrgProduction[labels.length];
      for(j=0; j<labels.length;j++){  //Prochazi jedno pole neuspechu
        prods[j]=null;
        for(k=0;k<productions.length;k++){//Hleda oznaceni mezi vsemi pravidly
          if(((PrgProduction)productions[k]).hasLabel(labels[j])){
            prods[j]=(PrgProduction)productions[k]; //Nalezene prida do pole
            break;                   //a pokracuje hledanim dalsiho
          }
        }
        if(prods[j]==null){  //Pozadovane oznaceni nema zadne pravidlo
          return labels[j];  //Vraci toto oznaceni
        }
      }
      ((PrgProduction)productions[i]).setFail(prods); //Nastavi pole pravidel
    }
    return null;
  }
}
